function geometry_problem
global data

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  SECTION 1: GEOMETRY OF THE PROBLEM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1) Domain Omega=[0,1]x[0,1] and the mesh of triangles
data.nx=10; data.ny=data.nx;
[p,t,id]=mesh_triangle(0,1,0,1,data.nx,data.ny); 
data.p=p; data.t=t; np=size(data.p,1);

% 2) Boundary nodes (for Dirichlet BC)
ID=union(union(union(id.left,id.right),id.bottom),id.upper);
% indices in vectors
nd=size(ID,1);
data.ind_dir=reshape([2*ID'-1;2*ID'],2*nd,1);
data.ind_rest=setdiff([1:2*np]',data.ind_dir);
data.ID=ID;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  SECTION 2: PDE PROBLEM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1) Operator
nu=0.5;                 % viscosity
alpha=0.25;             % at mass matrix
diffuse_op='lap_op';  % lap_op/symgr_op

% 2) Right hand-side function
% Experimental solution (analytic for homogenneous Dirichlet BC on [0,1]x[0,1])
u1e = @(x,y) -cos(2*pi*x).*sin(2*pi*y)+sin(2*pi*y);
u2e = @(x,y) sin(2*pi*x).*cos(2*pi*y)-sin(2*pi*x);
pe  = @(x,y) 2*pi*(cos(2*pi*y)-cos(2*pi*x));
% ... and its derivatives
u1ex  = @(x,y) 2*pi*sin(2*pi*x).*sin(2*pi*y);
u1exx = @(x,y) 4*pi^2*cos(2*pi*x).*sin(2*pi*y);
u1ey  = @(x,y) -2*pi*cos(2*pi*x).*cos(2*pi*y)+2*pi*cos(2*pi*y);
u1eyy = @(x,y) 4*pi^2*cos(2*pi*x).*sin(2*pi*y)-4*pi^2*sin(2*pi*y);
u1exy = @(x,y) 4*pi^2*sin(2*pi*x).*cos(2*pi*y);
u2ex  = @(x,y) 2*pi*cos(2*pi*x).*cos(2*pi*y)-2*pi*cos(2*pi*x);
u2exx = @(x,y) -4*pi^2*sin(2*pi*x).*cos(2*pi*y)+4*pi^2*sin(2*pi*x);
u2ey  = @(x,y) -2*pi*sin(2*pi*x).*sin(2*pi*y);
u2eyy = @(x,y) -4*pi^2*sin(2*pi*x).*cos(2*pi*y);
u2exy = @(x,y) -4*pi^2*cos(2*pi*x).*sin(2*pi*y);
pex  = @(x,y) 4*pi^2*sin(2*pi*x);
pey  = @(x,y) -4*pi^2*sin(2*pi*y);

x=p(:,1); y=p(:,2);
if strcmp(diffuse_op,'symgr_op')
    f1=-2*nu*(u1exx(x,y)+1/2*(u2exy(x,y)+u1eyy(x,y)))+u1e(x,y).*u1ex(x,y)+u2e(x,y).*u1ey(x,y)+pex(x,y)+alpha*u1e(x,y);
    f2=-2*nu*(1/2*(u2exx(x,y)+u1exy(x,y))+u2eyy(x,y))+u1e(x,y).*u2ex(x,y)+u2e(x,y).*u2ey(x,y)+pey(x,y)+alpha*u2e(x,y);
elseif strcmp(diffuse_op,'lap_op')
    f1=-nu*(u1exx(x,y)+u1eyy(x,y))+u1e(x,y).*u1ex(x,y)+u2e(x,y).*u1ey(x,y)+pex(x,y)+alpha*u1e(x,y);
    f2=-nu*(u2exx(x,y)+u2eyy(x,y))+u1e(x,y).*u2ex(x,y)+u2e(x,y).*u2ey(x,y)+pey(x,y)+alpha*u2e(x,y);
end
data.f(1:2:2*np-1,1)=f1; data.f(2:2:2*np,1)=f2;
data.nu=nu; data.alpha=alpha; data.diffuse_op=diffuse_op;
data.upe.u1e=u1e; data.upe.u2e=u2e; data.upe.pe=pe;

% 3) Boundary conditions (Dirichlet)
x=p(ID,1); y=p(ID,2);
d1=u1e(x,y); d2=u1e(x,y);
data.d(1:2:2*nd-1,1)=d1; data.d(2:2:2*nd,1)=d2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%