function [P,T,ID,IN]=mesh_triangle(min_x,max_x,min_y,max_y,nx,ny)
% square domain ... [min_x max_x]x[min_y max_y]
% nx,ny         ... number of elements in axes x and y
%
% P  ... column array [x,y] coordinates of nodes
% T  ... indices of vertices of triangles in the column array [index_1,index_2,index_3]
% ID ... indices of possible Dirichlet boundary points
% IN ... indices on possible Neumann boundary points (edges) + index on triangle
%
% Generate mesh of triangles
%
%         nx
%  ................. [max_x,max_y]
%  |\ |\ |
%  |_\|_\|
%  | /|\ |             ny
%  |/ |_\| .........
% [min_x,min_y]
%
% This stucture is modified in the first and the last triangle:
%   __
%  | /|
%  |/_|

hx=(max_x-min_x)/nx; hy=(max_y-min_y)/ny;
x=min_x:hx:max_x;    y=min_y:hy:max_y;
Nx=length(x);        Ny=length(y);

P=zeros(Nx*Ny,2);
T=zeros(2*(Nx-1)*(Ny-1),3);
ID.left=[]; ID.right=[]; ID.upper=[]; ID.bottom=[];
IN.left=[]; IN.right=[]; IN.upper=[]; IN.bottom=[];

nT=1; k=1;   % index of triangle, node
for j=1:Ny   % y-direction
  for i=1:Nx % x-direction
    if i==1,  ID.left   = [ID.left;k];   end
    if i==Nx, ID.right  = [ID.right;k];  end
    if j==1,  ID.bottom = [ID.bottom;k]; end
    if j==Ny, ID.upper  = [ID.upper;k];  end
    if i<Nx && j<Ny
      T(nT,:)=[k,k+1,k+Nx]; T(nT+1,:)=[k+1,k+Nx+1,k+Nx]; nT=nT+2; % bottom, upper triangel
      if i==1,    IN.left   = [IN.left;[k,k+Nx,nT-2]];       end 
      if i==Nx-1, IN.right  = [IN.right;[k+1,k+Nx+1,nT-1]];  end
      if j==1,    IN.bottom = [IN.bottom;[k,k+1,nT-2]];      end
      if j==Ny-1, IN.upper  = [IN.upper;[k+Nx,k+Nx+1,nT-1]]; end
    end   
    P(k,:)=[x(i),y(j)]; k=k+1;
  end
end

% Corrections (avoiding two edges on the boundary)
 TAUX=T(1,:); T(1,:)=T(2,:); T(1,1)=T(1,1)-1; T(2,:)=TAUX; T(2,3)=T(2,3)+1;
 TAUX=T(end-1,:); T(end-1,:)=T(end,:); T(end-1,1)=T(end-1,1)-1; T(end,:)=TAUX; T(end,3)=T(end,3)+1;
 IN.bottom(1,3)=IN.bottom(1,3)+1; IN.upper(end,3)=IN.upper(end,3)-1;

% %Plotting
% %1)
% figure
% triplot(T,P(:,1),P(:,2)), axis equal
 
 
% for i=1:size(P,1), text(P(i,1),P(i,2),num2str(i)), end, pause, close
% %2)
% for i=1:size(T,1),i
%      tt=T(1:i,:); figure
%      triplot(tt,P(:,1),P(:,2)), axis equal 
  %    pause, close
% end