clear all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SECTION 1: GENERATE PROBLEM (produces data structure)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global data
data.name='3D_exact1'; 
addpath(data.name);
time=cputime;
geometry_problem % GEOMETRY: p t ID nx ny --> data
                 % PROBLEM:  nu alpha diffuse_op upe --> data
                 %           f d --> data
disp(['Problem setting: ',num2str(cputime-time),'sec.'])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SECTION 2: TEST MATRICES, AGREEMENT OF VECTORIZED VERSUS NONVECTORIZED 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
w=rand(size(data.f,1),1);
if strcmp(data.diffuse_op,'lap_op'), disp('Test for lap_op, Oseen matrices!!!')
    time=cputime;
    [A_,Bu_,Bl_,E_,b_,c_,a_]=assemblyP1bP1C_3(data.p,data.t,data.nu,data.alpha,data.f,w);
    disp(['Matrices assembling: ',num2str(cputime-time),'sec.'])
    time=cputime;
    %[A,Bu,Bl,E,b,c,a]=assemblyP1bP1C_vec_3(data.p,data.t,data.nu,data.alpha,data.f,w);
    [A,Bu,Bl,E,b,c,a]=assembly3DP1bP1C_vecO(data.p,data.t,data.nu,data.alpha,data.f,w);
    disp(['Matrices assembling: ',num2str(cputime-time),'sec.'])
elseif strcmp(data.diffuse_op,'symgr_op'), disp('Test for symgr_op, Oseen matrices!!!')
    time=cputime;
    [A_,Bu_,Bl_,E_,b_,c_,a_]=assemblyP1bP1DC_3(data.p,data.t,data.nu,data.alpha,data.f,w);
    disp(['Matrices assembling: ',num2str(cputime-time),'sec.'])
    time=cputime;
    %[A,Bu,Bl,E,b,c,a]=assemblyP1bP1DC_vec_3(data.p,data.t,data.nu,data.alpha,data.f,w);
    [A,Bu,Bl,E,b,c,a]=assembly3DP1bP1DC_vecO(data.p,data.t,data.nu,data.alpha,data.f,w);
    disp(['Matrices assembling: ',num2str(cputime-time),'sec.'])
end

Matrices=[norm(full(A_-A)), norm(full(Bu_-Bu)), norm(full(Bl_-Bl)), norm(full(E_-E))]
Vectors=[norm(full(b_-b)), norm(full(c_-c)), norm(full(a_-a))]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%