function geometry_problem
global data

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  SECTION 1: GEOMETRY OF THE PROBLEM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1) Domain Omega=[0,1]x[0,1]x[0,1] and the mesh of tetrahedras
nx=8; ny=nx; nz=nx; % only even
ax=0; ay=0; az=0;
bx=1; by=1; bz=1;
[p,t,id,in]=mesh_tetrahedral_3D_cube(ax,bx,ay,by,az,bz,nx,ny,nz);
data.nx=nx; data.ny=ny; data.nz=nz;
data.ax=ax; data.ay=ay; data.az=az;
data.bx=bx; data.by=by; data.bz=bz;
data.p=p; data.t=t;

% 2.1) Boundary nodes - for Dirichlet BC
ID=union(union(id.upper,id.front),id.back);
data.ID=ID;

% 2.2) Boundary nodes - for Neumann BC [p1,p2,p3]
IN=[in.left(:,[1 2 3]);in.right(:,[1 2 3])];
data.IN=IN;

% 2.3) Boundary nodes - for stick-slip BC [[p1,p2,p3],p4]=[surface triangle,inner node]
IC=in.bottom;
data.IC=IC;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  SECTION 2: PDE PROBLEM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1) Operator
nu=1/2;                 % viscosity
alpha=0.25;                % at mass matrix
diffuse_op='symgr_op';  % lap_op/symgr_op

% 2) Right hand-side function
% Experimental solution (analytic for homogenneous Dirichlet-Neumann BC on [0,1]x[0,1]x[0,1])
u1e = @(x,y,z) 4*(1-cos(2*pi*x)).*sin(2*pi*y).*z.*(1-z);
u2e = @(x,y,z) 4*sin(2*pi*x).*(cos(2*pi*y)-1).*z.*(1-z);
u3e = @(x,y,z) z*0; % ZERO z-component
pe  = @(x,y,z) 2*pi*(-cos(2*pi*x)+2*cos(2*pi*y)-cos(2*pi*z));
% ... and its derivatives
u1ex  = @(x,y,z) 8*pi*sin(2*pi*x).*sin(2*pi*y).*z.*(1-z);
u1ey  = @(x,y,z) 8*pi*(1-cos(2*pi*x)).*cos(2*pi*y).*z.*(1-z);
u1ez  = @(x,y,z) 4*(1-cos(2*pi*x)).*sin(2*pi*y).*(1-2*z);
u1exx = @(x,y,z) 16*pi^2*cos(2*pi*x).*sin(2*pi*y).*z.*(1-z);
u1eyy = @(x,y,z) -16*pi^2*(1-cos(2*pi*x)).*sin(2*pi*y).*z.*(1-z);
u1exy = @(x,y,z) 16*pi^2*sin(2*pi*x).*cos(2*pi*y).*z.*(1-z);
u1ezz = @(x,y,z) -8*(1-cos(2*pi*x)).*sin(2*pi*y);
u1eyz = @(x,y,z) 8*pi*(1-cos(2*pi*x)).*cos(2*pi*y).*(1-2*z);
u1exz = @(x,y,z) 8*pi*sin(2*pi*x).*sin(2*pi*y).*(1-2*z);
u2ex  = @(x,y,z) 8*pi*cos(2*pi*x).*(cos(2*pi*y)-1).*z.*(1-z);
u2ey  = @(x,y,z) -8*pi*sin(2*pi*x).*sin(2*pi*y).*z.*(1-z);
u2ez  = @(x,y,z) 4*sin(2*pi*x).*(cos(2*pi*y)-1).*(1-2*z);
u2exx = @(x,y,z) -16*pi^2*sin(2*pi*x).*(cos(2*pi*y)-1).*z.*(1-z);
u2eyy = @(x,y,z) -16*pi^2*sin(2*pi*x).*cos(2*pi*y).*z.*(1-z);
u2exy = @(x,y,z) -16*pi^2*cos(2*pi*x).*sin(2*pi*y).*z.*(1-z);
u2ezz = @(x,y,z) -8*sin(2*pi*x).*(cos(2*pi*y)-1);
u2eyz = @(x,y,z) -8*pi*sin(2*pi*x).*sin(2*pi*y).*(1-2*z);
u2exz = @(x,y,z) 8*pi*cos(2*pi*x).*(cos(2*pi*y)-1).*(1-2*z);
u3ex  = @(x,y,z) z*0;
u3ey  = @(x,y,z) z*0;
u3ez  = @(x,y,z) z*0;
u3exx = @(x,y,z) z*0;
u3eyy = @(x,y,z) z*0;
u3exy = @(x,y,z) z*0;
u3ezz = @(x,y,z) z*0;
u3eyz = @(x,y,z) z*0;
u3exz = @(x,y,z) z*0;
pex   = @(x,y,z) 4*pi^2*sin(2*pi*x);
pey   = @(x,y,z) -8*pi^2*sin(2*pi*y);
pez   = @(x,y,z) 4*pi^2*sin(2*pi*z);

x=p(:,1); y=p(:,2); z=p(:,3); np=size(p,1);
if strcmp(diffuse_op,'symgr_op')
    f1=-2*nu*(u1exx(x,y,z)+1/2*(u1eyy(x,y,z)+u1ezz(x,y,z)+u2exy(x,y,z)+u3exz(x,y,z)))+pex(x,y,z)+alpha*u1e(x,y,z);
    f2=-2*nu*(u2eyy(x,y,z)+1/2*(u2exx(x,y,z)+u2ezz(x,y,z)+u1exy(x,y,z)+u3eyz(x,y,z)))+pey(x,y,z)+alpha*u2e(x,y,z);
    f3=-2*nu*(u3ezz(x,y,z)+1/2*(u3exx(x,y,z)+u3eyy(x,y,z)+u1exz(x,y,z)+u2eyz(x,y,z)))+pez(x,y,z)+alpha*u3e(x,y,z);
elseif strcmp(diffuse_op,'lap_op')
    f1=-nu*(u1exx(x,y,z)+u1eyy(x,y,z)+u1ezz(x,y,z))+pex(x,y,z)+alpha*u1e(x,y,z);
    f2=-nu*(u2exx(x,y,z)+u2eyy(x,y,z)+u2ezz(x,y,z))+pey(x,y,z)+alpha*u2e(x,y,z);
    f3=-nu*(u3exx(x,y,z)+u3eyy(x,y,z)+u3ezz(x,y,z))+pez(x,y,z)+alpha*u3e(x,y,z);
end
data.f(1:3:3*np-2,1)=f1; data.f(2:3:3*np-1,1)=f2; data.f(3:3:3*np,1)=f3;
data.nu=nu; data.alpha=alpha; data.diffuse_op=diffuse_op;
%data.upe.u1e=u1e; data.upe.u2e=u2e; data.upe.u3e=u3e; data.upe.pe=pe;

% 3.1) Boundary conditions - Dirichlet
x=p(ID,1); y=p(ID,2); z=p(ID,3); nd=size(ID,1);
d1=u1e(x,y,z); d2=u2e(x,y,z); d3=u3e(x,y,z);   
data.d=[];
data.d(1:3:3*nd-2,1)=d1; data.d(2:3:3*nd-1,1)=d2; data.d(3:3:3*nd,1)=d3;

% 3.2) Boundary conditions - Neumann
if strcmp(diffuse_op,'symgr_op')
    % normal vector [-1 0 0]
    x=p(in.left(:,1),1); y=p(in.left(:,1),2); z=p(in.left(:,1),3);
    s1=-2*nu*u1ex(x,y,z)+pe(x,y,z); s2=-nu*(u1ey(x,y,z)+u2ex(x,y,z)); s3=-nu*(u1ez(x,y,z)+u3ex(x,y,z));
    x=p(in.left(:,2),1); y=p(in.left(:,2),2); z=p(in.left(:,2),3);
    s1=[s1 -2*nu*u1ex(x,y,z)+pe(x,y,z)]; s2=[s2 -nu*(u1ey(x,y,z)+u2ex(x,y,z))]; s3=[s3 -nu*(u1ez(x,y,z)+u3ex(x,y,z))];
    x=p(in.left(:,3),1); y=p(in.left(:,3),2); z=p(in.left(:,3),3);
    s1=[s1 -2*nu*u1ex(x,y,z)+pe(x,y,z)]; s2=[s2 -nu*(u1ey(x,y,z)+u2ex(x,y,z))]; s3=[s3 -nu*(u1ez(x,y,z)+u3ex(x,y,z))];
    % normal vector [1 0 0]
    x=p(in.right(:,1),1); y=p(in.right(:,1),2); z=p(in.right(:,1),3);
    s1_=2*nu*u1ex(x,y,z)-pe(x,y,z); s2_=nu*(u1ey(x,y,z)+u2ex(x,y,z)); s3_=nu*(u1ez(x,y,z)+u3ex(x,y,z));
    x=p(in.right(:,2),1); y=p(in.right(:,2),2); z=p(in.right(:,2),3);
    s1_=[s1_ 2*nu*u1ex(x,y,z)-pe(x,y,z)]; s2_=[s2_ nu*(u1ey(x,y,z)+u2ex(x,y,z))]; s3_=[s3_ nu*(u1ez(x,y,z)+u3ex(x,y,z))];
    x=p(in.right(:,3),1); y=p(in.right(:,3),2); z=p(in.right(:,3),3);
    s1_=[s1_ 2*nu*u1ex(x,y,z)-pe(x,y,z)]; s2_=[s2_ nu*(u1ey(x,y,z)+u2ex(x,y,z))]; s3_=[s3_ nu*(u1ez(x,y,z)+u3ex(x,y,z))];
elseif strcmp(diffuse_op,'lap_op')
    % normal vector [-1 0 0]
    x=p(in.left(:,1),1); y=p(in.left(:,1),2); z=p(in.left(:,1),3);
    s1=-nu*u1ex(x,y,z)+pe(x,y,z); s2=-nu*u2ex(x,y,z); s3=-nu*u3ex(x,y,z);
    x=p(in.left(:,2),1); y=p(in.left(:,2),2); z=p(in.left(:,2),3);
    s1=[s1 -nu*u1ex(x,y,z)+pe(x,y,z)]; s2=[s2 -nu*u2ex(x,y,z)]; s3=[s3 -nu*u3ex(x,y,z)];
    x=p(in.left(:,3),1); y=p(in.left(:,3),2); z=p(in.left(:,3),3);
    s1=[s1 -nu*u1ex(x,y,z)+pe(x,y,z)]; s2=[s2 -nu*u2ex(x,y,z)]; s3=[s3 -nu*u3ex(x,y,z)];
    % normal vector [1 0 0]
    x=p(in.right(:,1),1); y=p(in.right(:,1),2); z=p(in.right(:,1),3);
    s1_=nu*u1ex(x,y,z)-pe(x,y,z); s2_=nu*u2ex(x,y,z); s3_=nu*u3ex(x,y,z);
    x=p(in.right(:,2),1); y=p(in.right(:,2),2); z=p(in.right(:,2),3);
    s1_=[s1_ nu*u1ex(x,y,z)-pe(x,y,z)]; s2_=[s2_ nu*u2ex(x,y,z)]; s3_=[s3_ nu*u3ex(x,y,z)];
    x=p(in.right(:,3),1); y=p(in.right(:,3),2); z=p(in.right(:,3),3);
    s1_=[s1_ nu*u1ex(x,y,z)-pe(x,y,z)]; s2_=[s2_ nu*u2ex(x,y,z)]; s3_=[s3_ nu*u3ex(x,y,z)];
end
data.s1=[s1;s1_]; data.s2=[s2;s2_]; data.s3=[s3;s3_];

% 3.3) Boundary conditions - stick-slip (Navier-Tresca type)
g_fce     = @(x,y,z) 500*ones(size(IC,1),1);
kappa_fce = @(x,y,z) 500*ones(size(IC,1),1);
x=p(IC(:,1),1); y=p(IC(:,1),2); z=p(IC(:,1),3);
data.gg=g_fce(x,y,z); data.kkpp=kappa_fce(x,y,z);
x=p(IC(:,2),1); y=p(IC(:,2),2); z=p(IC(:,2),3);
data.gg=[data.gg g_fce(x,y,z)]; data.kkpp=[data.kkpp kappa_fce(x,y,z)];
x=p(IC(:,3),1); y=p(IC(:,3),2); z=p(IC(:,3),3);
data.gg=[data.gg g_fce(x,y,z)]; data.kkpp=[data.kkpp kappa_fce(x,y,z)];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%