function [p,t,id,in]=mesh_tetrahedral_3D_cube(ax,bx,ay,by,az,bz,nx,ny,nz)

%  z
%  |
%  | 
%  |    /y
%  |   /
%  |  /
%  | /
%  |/_______________x

addpath('../iso2mesh')

hx=(bx-ax)/nx; hy=(by-ay)/ny;  hz=(bz-az)/nz; 
x_scale=ax:hx:bx; y_scale=ay:hy:by; z_scale=az:hz:bz;
[p,t]=meshgrid5(x_scale,y_scale,z_scale);

rmpath('../iso2mesh')

% Dirichlet boundary nodes
id.left=find(p(:,1)==ax);   id.right=find(p(:,1)==bx);
id.front=find(p(:,2)==ay);  id.back=find(p(:,2)==by);
id.bottom=find(p(:,3)==az); id.upper=find(p(:,3)==bz);

% Neumann boundary elements [p1 p2 p3 p4]
if nargout==4
    ind4=(p(t(:,1),1)==ax)&(p(t(:,2),1)==ax)&(p(t(:,3),1)==ax);
    in.left=[t(ind4,:)];
    ind3=(p(t(:,1),1)==ax)&(p(t(:,2),1)==ax)&(p(t(:,4),1)==ax);
    in.left=[in.left;t(ind3,[1,2]),t(ind3,4),t(ind3,3)];
    ind2=(p(t(:,1),1)==ax)&(p(t(:,3),1)==ax)&(p(t(:,4),1)==ax);
    in.left=[in.left;t(ind2,1),t(ind2,[3,4]),t(ind2,2)];
    ind1=(p(t(:,2),1)==ax)&(p(t(:,3),1)==ax)&(p(t(:,4),1)==ax);
    in.left=[in.left;t(ind1,2:4),t(ind1,1)];

    ind4=(p(t(:,1),1)==bx)&(p(t(:,2),1)==bx)&(p(t(:,3),1)==bx);
    in.right=[t(ind4,:)];
    ind3=(p(t(:,1),1)==bx)&(p(t(:,2),1)==bx)&(p(t(:,4),1)==bx);
    in.right=[in.right;t(ind3,[1,2]),t(ind3,4),t(ind3,3)];
    ind2=(p(t(:,1),1)==bx)&(p(t(:,3),1)==bx)&(p(t(:,4),1)==bx);
    in.right=[in.right;t(ind2,1),t(ind2,[3,4]),t(ind2,2)];
    ind1=(p(t(:,2),1)==bx)&(p(t(:,3),1)==bx)&(p(t(:,4),1)==bx);
    in.right=[in.right;t(ind1,2:4),t(ind1,1)];

    ind4=(p(t(:,1),2)==ay)&(p(t(:,2),2)==ay)&(p(t(:,3),2)==ay);
    in.front=[t(ind4,:)];
    ind3=(p(t(:,1),2)==ay)&(p(t(:,2),2)==ay)&(p(t(:,4),2)==ay);
    in.front=[in.front;t(ind3,[1,2]),t(ind3,4),t(ind3,3)];
    ind2=(p(t(:,1),2)==ay)&(p(t(:,3),2)==ay)&(p(t(:,4),2)==ay);
    in.front=[in.front;t(ind2,1),t(ind2,[3,4]),t(ind2,2)];
    ind1=(p(t(:,2),2)==ay)&(p(t(:,3),2)==ay)&(p(t(:,4),2)==ay);
    in.front=[in.front;t(ind1,2:4),t(ind1,1)];

    ind4=(p(t(:,1),2)==by)&(p(t(:,2),2)==by)&(p(t(:,3),2)==by);
    in.back=[t(ind4,:)];
    ind3=(p(t(:,1),2)==by)&(p(t(:,2),2)==by)&(p(t(:,4),2)==by);
    in.back=[in.back;t(ind3,[1,2]),t(ind3,4),t(ind3,3)];
    ind2=(p(t(:,1),2)==by)&(p(t(:,3),2)==by)&(p(t(:,4),2)==by);
    in.back=[in.back;t(ind2,1),t(ind2,[3,4]),t(ind2,2)];
    ind1=(p(t(:,2),2)==by)&(p(t(:,3),2)==by)&(p(t(:,4),2)==by);
    in.back=[in.back;t(ind1,2:4),t(ind1,1)];

    ind4=(p(t(:,1),3)==az)&(p(t(:,2),3)==az)&(p(t(:,3),3)==az);
    in.bottom=[t(ind4,:)];
    ind3=(p(t(:,1),3)==az)&(p(t(:,2),3)==az)&(p(t(:,4),3)==az);
    in.bottom=[in.bottom;t(ind3,[1,2]),t(ind3,4),t(ind3,3)];
    ind2=(p(t(:,1),3)==az)&(p(t(:,3),3)==az)&(p(t(:,4),3)==az);
    in.bottom=[in.bottom;t(ind2,1),t(ind2,[3,4]),t(ind2,2)];
    ind1=(p(t(:,2),3)==az)&(p(t(:,3),3)==az)&(p(t(:,4),3)==az);
    in.bottom=[in.bottom;t(ind1,2:4),t(ind1,1)];

    ind4=(p(t(:,1),3)==bz)&(p(t(:,2),3)==bz)&(p(t(:,3),3)==bz);
    in.upper=[t(ind4,:)];
    ind3=(p(t(:,1),3)==bz)&(p(t(:,2),3)==bz)&(p(t(:,4),3)==bz);
    in.upper=[in.upper;t(ind3,[1,2]),t(ind3,4),t(ind3,3)];
    ind2=(p(t(:,1),3)==bz)&(p(t(:,3),3)==bz)&(p(t(:,4),3)==bz);
    in.upper=[in.upper;t(ind2,1),t(ind2,[3,4]),t(ind2,2)];
    ind1=(p(t(:,2),3)==bz)&(p(t(:,3),3)==bz)&(p(t(:,4),3)==bz);
    in.upper=[in.upper;t(ind1,2:4),t(ind1,1)];
end